# Generated by Django 5.2.5 on 2025-09-03 10:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ixp_tracker', '0017_ixp_org_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='ixp',
            name='anchor_host',
            field=models.BooleanField(default=False),
        ),
    ]
